package util;

import org.apache.commons.codec.digest.DigestUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

public class CreamApiDllHandler {

    private static CreamApiDllHandler creamApiDllHandlerInstance;
    private final Path steamApiDllPath = Paths.get("steam_api.dll");
    private final Path steamApi64DllPath = Paths.get("steam_api64.dll");
    private final String steamApiDllMd5;
    private final String steamApi64DllMd5;

    private CreamApiDllHandler() {
        String steamApiDllMd5 = "";
        String steamApi64DllMd5 = "";
        try {
            steamApiDllMd5 = DigestUtils.md5Hex(Files.newInputStream(steamApiDllPath));
        } catch (IOException e) {
            System.err.println("File missing: " + steamApiDllPath.toAbsolutePath().toString());
        }
        try {
            steamApi64DllMd5 = DigestUtils.md5Hex(Files.newInputStream(steamApi64DllPath));
        } catch (IOException e) {
            System.err.println("File missing: " + steamApi64DllPath.toAbsolutePath().toString());
        }
        this.steamApiDllMd5 = steamApiDllMd5;
        this.steamApi64DllMd5 = steamApi64DllMd5;
    }

    public static synchronized CreamApiDllHandler getInstance() {
        if (creamApiDllHandlerInstance == null) {
            creamApiDllHandlerInstance = new CreamApiDllHandler();
        }
        return creamApiDllHandlerInstance;
    }

    public Path getDllPath(boolean is64Bit) {
        return is64Bit ? steamApi64DllPath : steamApiDllPath;
    }

    public String getDllMd5(boolean is64Bit) {
        return is64Bit ? steamApi64DllMd5 : steamApiDllMd5;
    }
}
