package util;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import kong.unirest.HttpResponse;
import kong.unirest.Unirest;
import pojo.App;
import pojo.Download;
import pojo.SteamAppsList;

import java.io.*;
import java.lang.reflect.Type;
import java.time.Duration;
import java.time.Instant;
import java.util.List;
import java.util.stream.Collectors;

public class SteamAppsListCache {

    private SteamAppsList list = new SteamAppsList();
    private File cacheFile = new File("steamapps.json");

    public SteamAppsListCache() {
        boolean fileFound = true;
        try {
            getListFromFile();
        } catch (FileNotFoundException e) {
            System.err.println("File does not exist, trying to create steamapps.json for the first time...");
            fileFound = false;
        }
        if (!fileFound || Instant.now().isAfter(list.getTimestamp().plus(Duration.ofDays(3)))) {
            sync();
        }
    }

    private void sync() {
        getListFromApi();
        try {
            saveListToFile();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    public App getGame(int appId) {
        return list.getSteamAppsList().stream().filter(app -> app.getAppId() == appId).findFirst().orElse(null);
    }

    @SuppressWarnings("unused")
    public App getGame(String name) {
        return list.getSteamAppsList().stream()
                .filter(app -> app.getName().equalsIgnoreCase(name)).findFirst().orElse(null);
    }

    public App findGame(String name) {
        for (App app : list.getSteamAppsList()) {
            if (app.getName().toLowerCase().replaceAll("[^a-zA-Z0-9\\s+]", "")
                    .startsWith(name.toLowerCase().replaceAll("[^a-zA-Z0-9\\s+]", ""))) {
                return app;
            }
        }
        for (App app : list.getSteamAppsList()) {
            if (app.getName().toLowerCase().replaceAll("[^a-zA-Z0-9\\s+]", "")
                    .contains(name.toLowerCase().replaceAll("[^a-zA-Z0-9\\s+]", ""))) {
                return app;
            }
        }
        return null;
    }

    private void getListFromApi() {
        HttpResponse<String> httpResponse =
                Unirest.get("https://api.steampowered.com/IStoreService/GetAppList/v1/" +
                        "?key=E427256C579D3CDF1D504810E8F5B948&include_games=1&max_results=50000").asString();
        List<App> apps = new Gson()
                .fromJson(httpResponse.getBody(), Download.class)
                .getResponse().getApps();
        list.setTimestamp(Instant.now());
        list.setSteamAppsList(apps);
    }

    private void saveListToFile() throws IOException {
        Gson gson = new Gson();
        String jsonString = gson.toJson(list);
        BufferedWriter fOut = new BufferedWriter(new FileWriter(cacheFile));
        fOut.write(jsonString);
        fOut.close();
    }

    private void getListFromFile() throws FileNotFoundException {
        BufferedReader fIn = new BufferedReader(new FileReader(cacheFile));
        String json = fIn.lines().collect(Collectors.joining());
        final Type type = new TypeToken<SteamAppsList>() {
        }.getType();
        Gson gson = new Gson();
        list = gson.fromJson(json, type);
    }
}
